//
// PIC16F886 8MHz
// Human detect sensor using IR Distance Sensor
//
//
//  2011-MAR-15  Ver2.1   Y.Fujioka
//	2012-JAN-15  Ver3.0   Y.Fujioka
//
//	Port Mapping
//  
//	RA0		RS			O
//	RA1		DCN			I
//	RA2		KEY_VREF	A
//	RA3		ENBL		O
//	RA4		D4			O
//	RA5		D5			O
//	RA6		D6			O
//	RA7		D7			O
//	RB0		TRIG_OUT	O
//	RB1		LED1		O
//	RB2		LED2		O
//	RB3		KEYIN		A
//	RB4		SENSOR2		A
//	RB5		SENSOR1		A
//	RB6		ICSPCLK		O
//	RB7		ICSPDAT		O
//	RC0		XTAL2		I
//	RC1		XTAL1		I
//	RC2		NC			O
//	RC3		ON_5V		O
//	RC4		VSEL2		O
//	RC5		VSEL1		O
//	RC6		TXD			O
//	RC7		RXD			I
//	RE3		RST			I

//
//  TIMER1  32.768KHz XTAL for 1sec interrupt
//  USe Comparator INT for KEY Detection
//	KEY1 = 0V		0 %
//  KEY2 = 1.40V	50%
//  KEY3 = 1.86V	66%
//  KEY_VREF = 2.3V  82%
//
// Ver2.0  2011-02-20  
// Verion \ǉ
// Lo-Batt ɂ̓J_ȊO̓~A2.5V ȏ̓dmŕAiqXeVXjǉ
//
//  Ver2.1  2011-03-15
//  SLEEP31KHz NbNɕύX
//  A[̃pXЂ2bɕύX
//
//  Ver2.3 2011-04-27 Sleep ̃NbN8MHz ɖ߂i31KƎvɒx邽߁j
//
//  Ver2.4 2011-05-05 ݒ莞ɒmȂ悤ɕύX
// @ver2.5               
//   u]|vmȂBUGBASYύX
//   uNvu]|vmɂPbŁus݁vɂȂĂ܂C
//   obe`FbNAAl̏ԌmAA[ꂼʃ^XNƂ	 	
//
//   Ver3.0 @2012-JAN-15
//   L[XpɃRg
//

/** INCLUDES *******************************************************/

#include <16f886.h>
#include <stdlib.h>

#fuses INTRC_IO, NOWDT, NOPUT, NOBROWNOUT, NOPROTECT
#use delay(clock=8000000)
#use RS232(BAUD=9600, XMIT=PIN_C6, RCV=PIN_C7, PARITY=N, BITS=8)

#use fast_io(A)
#use fast_io(B)
#use fast_io(C)

/** CONFIGURATION **************************************************/

#BYTE TRISA = 0x85
#BYTE TRISB = 0x86
#BYTE TRISC = 0x87   
#BYTE PORTA = 0x05
#BYTE PORTB = 0x06
#BYTE PORTC = 0x07
#BYTE INTCON = 0x8b
#BYTE PIR1 = 0x0c
#BYTE PIR2 = 0x0d
#BYTE PIE1 = 0x8c
#BYTE PIE2 = 0x8d
#BYTE OSCCON = 0x8f
#BYTE ANSEL = 0x11e
#BYTE ANSELH = 0x11f
#BYTE ADCON0 = 0x1f
#BYTE ADCON1 = 0x9f
#BYTE ADRESH = 0x1e
#BYTE ADRESL = 0x9e
#BYTE OPTION_REG = 0x81
#BYTE T1CON = 0x10
#BYTE CCP1CON = 0x17
#BYTE PR2 = 0x92
#BYTE CCPR1L = 0x15
#BYTE T2CON = 0x12
#BYTE TMR1H = 0x0f
#BYTE TMR1L = 0x0e
#BYTE TMR2 = 0x11
#BYTE OSCTUNE = 0x90
#BYTE SRCON = 0x185
#BYTE EEDAT = 0x10C
#BYTE EEADR = 0x10D
#BYTE EECON1 = 0x18C
#BYTE EECON2 = 0x18D
#BYTE TMR0 = 0x01

#BIT KEY_VREF = ANSEL.2		// ANS2
#BIT KEY_IN = ANSELH.1		// ANS9
#BIT SENSOR2 = ANSELH.3		// ANS11 
#BIT SENSOR1 = ANSELH.5		// ANS13
#BIT ADON = ADCON0.0
#BIT GODONE = ADCON0.1
#BIT CHS3 = ADCON0.5
#BIT CHS2 = ADCON0.4
#BIT CHS1 = ADCON0.3
#BIT CHS0 = ADCON0.2
#BIT FVREN = SRCON.0
#BIT VCFG0 = ADCON1.4
#BIT VCFG1 = ADCON1.5
#BIT ADFM = ADCON1.7
#BIT ADCS1 = ADCON0.7
#BIT ADCS0 = ADCON0.6
#BIT EEPGD = EECON1.7
#BIT WRERR = EECON1.3
#BIT WREN = EECON1.2
#BIT EEWR = EECON1.1
#BIT EERD = EECON1.0
#BIT TMR1ON = T1CON.0

#BIT RS 	= PORTA.0
#BIT DCIN 	= PORTA.1
#BIT DBG_OUT	= PORTA.2
#BIT ENBL	= PORTA.3
#BIT LED1	= PORTB.2
#BIT LED2 	= PORTB.1
#BIT LED3   = PORTB.0
#BIT VSEL1	= PORTC.5
#BIT VSEL2	= PORTC.4
#BIT TRIG_OUT = PORTC.3
#BIT LCD_BKLGT = PORTC.2

#BIT INTEDG = OPTION_REG.6
#BIT TMR2ON = T2CON.2
#BIT PEIE = INTCON.6
#BIT GIE = INTCON.7
#BIT TMR1IF = PIR1.0
#BIT TMR1IE = PIE1.0


#define UPPER_LIMIT 0
#define DIST_MIN 0
#define DIST_MAX 0
#define ALARM_LIMIT 0
#define RESPONSE_LMT 0

#define OPEN 0
#define SET  1
#define SELECT 2
#define CHANGE 3 


/** V A R I A B L E S ********************************************************/

int i, counter_16Hz, int_1s_flag, int_500ms_flag, int_16Hz_flag, disp_flag,key_counter;
int temp, key_flag, blink_flag,sensor_flag, state, next_state;
int key, blink_buf_flag, blink_start, blink_number, blink_on;
int sec, min, hour, youbi;
int disp_mode, joutai, alarm_enable, alarm_min_flag;
int bc_state, bat_chk_flag, dc_state, dist_chk_flag, ag_state;

unsigned int dist_cm;
long dist, vRef, vSensor, vBatt, vKey, voltage, temp_long1, temp_long2, temp_long3;
char LCDText[16*2+1];
char blink_buf[33];
char tempbuf[4];
int yotei_hour_start, yotei_min_start, yotei_hour_stop, yotei_min_stop, yotei_sw;
int yotei_youbi, yotei_time;
int sensor_sw1, sensor_sw2, sensor_sw3, sensor_sw4, detect_time;
int dist1, dist2, detect1, detect2, detect3;
int S1_detect, S2_detect;
int chakuza_sec, chakuza_min, alarm_time, alarm_sw1, alarm_sw2, alarm_sw3, alarm_sw4 ;
int  hd_state, joutai_old, chakuza_flag; 
unsigned int hd_timer;
int Trig_Out_counter, Lo_Batt_flag;
static char version[] = "Ver2.5";

int debug_flag;

/** P R I V A T E  P R O T O T Y P E S ***************************************/
void	LCDInit(void);
void	LCDErase(void);
void 	LCDUpdate(void);
void 	LCDWrite(char cd, char Data);

void param_init(void);
void adc_init(void);
void send_alarm(int data);
unsigned int cal_dist1(long voltage);
unsigned int cal_dist2(long voltage);
long read_sensor(int number);
void calendar(void);
void param_set(void);
void NormalDisp(void);
void LCDDisp(void);
void blink(void);
int read_key(void);
void youbi_set(int line);
void yotei_1_set(void);
void yotei_2_set(void);
void yotei_3_set(void);

void batt_check(void);				// obed`FbN
void dist_check(void);				// ZT[vZ
void human_detect(void);				// l̃ZT[m
void alarm_gen(void);				// A[


int read_e2rom(int addr);
void write_e2rom(int addr, int data);
void read_settei(int youbi, int time);
void write_settei(int youbi, int time);
void alarm_time_check(void);
void write_dist_settei(void);
void read_dist_settei(void);
void dist_1_set();				
void dist_2_set();
void gen_alarm(void);
//void battery_check(void);
void init_e2prom_settei(void);

/** VECTOR REMAPPING ***********************************************/
	
#INT_TIMER0
void int_tmr0()	
{
}


#INT_TIMER1
void int_tmr1()		// 16Hz/ 62.5mS  Timer
{
	TMR1H = 0xF8;		//  NbN@16Hz 
	TMR1L = 0x01;		// ǂ킯8bit ƎvxĂ܂BBB

	if(debug_flag == 1)
		{
		DBG_OUT = 1;		// fobOpɃ|[gp^p^
		debug_flag = 0;
		}
	else
		{
		DBG_OUT = 0;
		debug_flag = 1;
		}		
	counter_16Hz++;
	key_counter ++;
	if (counter_16Hz == 8)
		{
		int_500ms_flag = 1;			// 0.5bƂ̃tO
		}
	else if (counter_16Hz == 16)	
		{
		counter_16Hz = 0;
		int_500ms_flag = 1;
		sensor_flag = 1;		// 1@bƂɃZT[`FbNp̃tO𗧂Ă
		dist_chk_flag = 1;
		bat_chk_flag = 1;
		int_1s_flag = 1;
		sec++;
		}
	int_16Hz_flag = 1;
}


#INT_TIMER2
void int_tmr2()
{
}

#INT_COMP2
void int_cmp2()
{
}

 

/** DECLARATIONS ***************************************************/

void main(void)
{  
    TRISA = 0b00000010;
	TRISB = 0b00111000;
	TRISC = 0b10000011;

	OSCCON = 0b01110001;	// 8MHz / Internal osc
	OPTION_REG = 0b10000111; 	// Setup Timer0 as Internal 8MHz clock with 1:256 prescaler  
	T1CON =  0b10001111;	// Setup Timer1 as 32.768KHz external xtal with 1:1 prescaler 
//	T1CON =  0b10001011;	// Setup Timer1 as 32.768KHz external xtal with 1:1 prescale synchronized

	param_init();

	enable_interrupts(INT_TIMER1);
	enable_interrupts(GLOBAL);	

	TMR1H = 0xF8;			// Setup T1 as 16Hz Timer
	TMR1L = 0x01;

	init_e2prom_settei();
	alarm_time_check();
	printf("START!\r\n");

	adc_init();
	LCDInit();
	LCDErase();
	memcpy(LCDText, "̼ï ", 13);
	memcpy(LCDText+16, "Ĳھݻ", 7);
	LCDUpdate();
	delay_ms(2000);		// \pɂQb҂

	while(1)		// Main Loop
		{
		if(int_16Hz_flag ==1)			// 16Hz Ń^XN
			{
			param_set();				// L[ɂݒp[^ύX
			batt_check();				// obed`FbN
			dist_check();				// ZT[vZ
			human_detect();				// l̃ZT[m
			alarm_gen();				// A[

			if (int_1s_flag == 1)		// If 1 sec then increment calendar
				{
				calendar();						// J_CNg
				NormalDisp();					// J_eLCDobt@ɓWJ
				alarm_time_check();				// A[ݒ莞ԓł邩̊mF
				}
			blink();					// _Őݒ莞2Hz _ŏ
			LCDDisp();					// LCD ɕ\{_Ńobt@
			}
//		OSCCON = 0b00000001;			// OSC = 31KHz Set ɂƂƎvxBBBRgAEg 
		sleep();	
		}
}


void param_init(void)
{
	VSEL1 = 0;
	VSEL2 = 0;

	LED1 = 0;
	LED2 = 0;
	LED3 = 0;
	TRIG_OUT = 0;
	sec = 0;
	min = 0;
	hour = 0;
	youbi = 0;
	state = 0;
	bc_state = 0;
	dc_state = 0;
	ag_state = 0;
	hd_state = 0;

	int_1s_flag = 0;
	bat_chk_flag = 0;
	dist_chk_flag = 0;
	
	disp_flag = 0;
	key_flag = 0;
	blink_on = 0;
	disp_mode = 0;
	yotei_hour_start = 0;
	yotei_min_start = 0;
	yotei_hour_stop = 0;
	yotei_min_stop = 0;
	yotei_sw = 0;
	yotei_youbi = 0;
	yotei_time = 0;
	sensor_sw1 = 0;
	sensor_sw2 = 0;
	sensor_sw3 = 0;
	sensor_sw4 = 0;
	detect_time = 1;
	alarm_min_flag = 0;
	chakuza_flag = 0;

	hd_timer = 0;
}


void init_e2prom_settei(void)
{
	int i, j;

	detect1 = 50;
	detect2 = 100;
	detect3 = 40;
	write_dist_settei();

	yotei_hour_start = 0;	
	yotei_min_start = 0;
	yotei_hour_stop = 23;
	yotei_min_stop = 59;
	yotei_sw = 1;
	sensor_sw1 = 0;
	sensor_sw2 = 1;
	sensor_sw3 = 1;
	sensor_sw4 = 1;
	detect_time = 10;

	for (i = 0; i< 7; i++)
		{
		for (j = 0; j< 5; j++)
			{
			write_settei(i, j);
			}
		}

	yotei_hour_stop = 1;		// Debug Sun Timer1 ̂ 0:00 - 01:00 ܂ Cl[u
	yotei_min_stop = 0;
	write_settei(0, 0);
}


void batt_check(void)			// dmXe[g}V
{
		switch(bc_state)
			{
			case 0:
				if(bat_chk_flag == 1)					// 1bƂɃobe`FbNXe[g}VN
					{
					bat_chk_flag = 0;
//					printf("bat Check\r\n");
					VSEL1 = 0;
					VSEL2 = 0;
					CHS3 = 1;							// Read Vref Voltage
					CHS2 = 1;
					CHS1 = 1;
					CHS0 = 1;
					delay_ms(1);
					GODONE = 1;		// ADC start
					while (GODONE == 1);
					vRef = ADRESH;	
					vRef = (vRef << 8) | ADRESL;		// reference voltage (0.6V) value
					vBatt = 1024*6/vRef;
//					printf("vBatt = %ld\r\n", vBatt);
					bc_state = 0;
					}
				break;
			default:
				break;
			}
}


void dist_check(void)				// Xe[g}V
	{
		switch(dc_state)
			{
			case 0:
				if(dist_chk_flag == 1)					// 1bƂɑ`FbNXe[g}VN
					{
					dist_chk_flag = 0;
//					printf("dist_chk\r\n");	
					VSEL1 = 1;		// Sensor 1 ON		
					VSEL2 = 0;
					dc_state = 1;
					}
				else
					{
					dc_state = 0;
					}
				break;

			case 1:
				CHS3 = 1;		// ANS13	Sensor 1 Read
				CHS2 = 1;
				CHS1 = 0;
				CHS0 = 1;	
				GODONE = 1;								//	ADC Start
				while(GODONE == 1);
				vSensor = ADRESH;
				vSensor = (vSensor << 8) | ADRESL;		// Read Sensor1 Voltage value	
				voltage = vSensor * 6* 8 / vRef;
//				printf("voltage = %lu\r\n", voltage);
				dist1 = cal_dist1(voltage);				// Calculate Sensor1 distance 
//				printf("dist1 = %u\r\n", dist1);
				VSEL1 = 0;		// Sensor 2 ON
				VSEL2 = 1;
				dc_state = 2;
				break;
		
			case 2:
				CHS3 = 1;		// ANS11	Sensor 2 Read
				CHS2 = 0;
				CHS1 = 1;
				CHS0 = 1;
				GODONE = 1;		// ADC Start
				while(GODONE == 1);
				vSensor = ADRESH;
				vSensor = (vSensor << 8) | ADRESL;		// Sensor Voltage value	
				VSEL1 = 0;		// Sensor 1 OFF
				VSEL2 = 0;		// Sensor 2 OFF		
				voltage = vSensor * 6 * 8 / vRef;
				dist2 = cal_dist2(voltage);	

//				printf("dist2 = %u\r\n", dist2);		
				dc_state = 0;
				break;

			default:
				break;
			}
		}


void human_detect(void)			//ľm
{
	switch (hd_state)
		{
		case 0:
			if(sensor_flag == 1)
				{
				sensor_flag = 0;
//				printf("human_detect\r\n");

				if (dist1 < detect1)  		// check if valid distance
					{
					S1_detect = 1;			// ZT[P@ԁ@
					LED1 = 1;
					LED2 = 0;
					}
				else if((dist1 >= detect1) && (dist1 < detect2))
					{
					S1_detect = 2;			// ZT[P@Nԁ@
					LED1 = 0;
					LED2 = 1;
					}
				else
					{
					S1_detect = 0;			// ZT[P@sݏ
					LED1 = 0;
					LED2 = 0;
					}
				if (dist2 < detect3)		// ZT[Q@ZT[@m
					{
					S2_detect = 1;
					LED3 = 1;
					}
				else						
					{
					S2_detect = 0;			// ZT[Q@sݏ
					LED3 = 0;
					}
				hd_state = 1;
				}			
			else
				{
				hd_state = 0;
				}			
			break;			
	
		case 1:
			if((S1_detect == 1) && (S2_detect == 1))	// 
				{
					chakuza_flag = 1;	// NmpɒԃtOZbg
					chakuza_sec ++;
					printf("chakuza_sec = %d\r\n", chakuza_sec);
					if (chakuza_sec == 60)
						{
						chakuza_sec = 0;
						chakuza_min ++;
						}
					if (chakuza_min >= alarm_time)		// x񎞊ԂɒBH
						{
						joutai = 2;		// Ԓ
						}
					else
						{
						joutai = 1;			// 
						}
					hd_state = 0;
					}
				else if((S1_detect == 2)&&(chakuza_flag ==1))
					{
					joutai = 3;			// N
//					chakuza_flag = 0;
					chakuza_sec = 0;
					chakuza_min = 0;
					hd_state = 0;
					}
				else if((S1_detect == 0) && (S2_detect == 1) &&(chakuza_flag ==1))
					{
					joutai = 4;			// ]|
//					chakuza_flag = 0;
					chakuza_sec = 0;
					chakuza_min = 0;
					hd_state = 0;
					}	
				else
					{
					joutai = 0;			// s
					chakuza_flag = 0;
					chakuza_sec = 0;
					chakuza_min = 0;
					hd_state = 0;
					}		
//				printf("joutai = %d\r\n", joutai);
			break;
	
		default:
			break;
		}
}


void alarm_gen(void)
{
	switch(ag_state)
		{
		case 0:
			if(alarm_enable == 1)		// xݒ莞ԓłΊeԂ̋󋵂ɉČx𔭐
				{
				if ((joutai == 1 && alarm_sw1 == 1) 
				|| (joutai == 2 && alarm_sw2 == 1)
				|| (joutai == 3 && alarm_sw3 == 1)
				|| (joutai == 4 && alarm_sw4 == 1)
				|| (joutai == 5))	
					{
					if (joutai != joutai_old)		// Ԃɕω΃A[
						{
						TRIG_OUT = 1;				// TRIG OUT ON
						Trig_Out_counter = 0;
						printf("ALARM!!\r\n");
						ag_state = 1;
						}
					else
						{
						ag_state = 0;
						}						
					}
				else
					{
					ag_state = 0;
					}
				}
			else
				{
				ag_state = 0;
				}
			joutai_old = joutai;
			break;

		case 1:
			Trig_Out_counter ++;
			if (Trig_Out_counter == 16)		// 1bԃ^C}[҂
				{
				TRIG_OUT = 0;
				Trig_Out_counter = 0;
				ag_state = 0;
				}
			else
				{
				ag_state = 1;
				}
			break;
		
		default:
			break;
	}
}
	



unsigned int cal_dist1(long voltage)
{
	if (voltage >= 25* 8)					// `ߎ@ixmۂ̂߁AW{ČvZĂj
		{
		dist = 20 * 8;
		}
	else if ((voltage >= 15* 8) && (voltage < 25* 8))
		{
		dist = -2 * voltage + 70 * 8;
		}
	else if ((voltage >= 10*8) && (voltage < 15*8))
		{
		dist = -4 * voltage + 100 * 8;
		}
	else if ((voltage >= 5*8) && (voltage < 10*8))
		{
		dist = -14 * voltage + 200 * 8;
		}
	else
		{
		dist = 130 * 8;
		}
	dist_cm = dist/ 8;
	return dist_cm;
}
 
unsigned int cal_dist2(long voltage)
{
	if (voltage >= 25 * 8)						// `ߎ
		{
		dist = 10 *8;
		}
	else if ((voltage >= 15 * 8) && (voltage < 25 * 8))
		{
		dist = -1 * voltage + 33* 8;
		}
	else if ((voltage >= 10* 8) && (voltage < 15 * 8))
		{
		dist = -2 * voltage + 48* 8;
		}
	else if ((voltage >= 5 * 8) && (voltage < 10 * 8))
		{
		dist = -6 * voltage + 88 * 8;
		}
	else if ((voltage >= 4 * 8) && (voltage < 5 * 8))
		{
		dist = -22 * voltage + 168 * 8;
		}
	else
		{
		dist = 80 * 8;
		}
	dist_cm = dist/ 8;

	return dist_cm;
}
 


void alarm_time_check(void)			// 1bƂɃ`FbNĎԓIȃA[m
{
	long hour_long, min_long;		// int ̃f[^vZLong ɂƂ̓f[^Long ɃRs[ĂvZȂƃ_I

	if(disp_mode == 0)				// Normal \̂ƂA[`FbNBiӰޕύX͋֎~j
		{
		alarm_enable = 0;			// ftHg̓A[ݒ肵ȂɂĂ
		hour_long = hour;
		min_long = min;
		temp_long1 = hour_long * 60 + min_long;		// ݂̎ԂvZ
	
		for(i = 0; i< 5; i++)		// 5̃^C][ɂЂĂ邩mF
			{
			read_settei(youbi, i);

			hour_long = yotei_hour_start;
			min_long = yotei_min_start;
			temp_long2 = hour_long * 60 + min_long;	

			hour_long = yotei_hour_stop;
			min_long = yotei_min_stop;
			temp_long3 = hour_long * 60 + min_long;	
			if ((temp_long1 >= temp_long2) && (temp_long1 < temp_long3))
				{
				if (yotei_sw == 1)
					{
					alarm_enable = 1;
					alarm_sw1 = sensor_sw1;		// ݎԂɊYA[XCb`Rs[
					alarm_sw2 = sensor_sw2;
					alarm_sw3 = sensor_sw3;
					alarm_sw4 = sensor_sw4;
					alarm_time = detect_time;
					}
				}
			}
		}
}
				
		


void param_set(void)
{
	int_16Hz_flag = 0;
	key = read_key();						// L[̒lǂݍŉs
	switch(state)
		{
		case 0:
			if(key == SET)
				{
				key_counter = 0;
				state = 1;
				}
			else
				{
				disp_mode = 0;
				state = 0;
				}
			break;
	
		case 1:
			if(key == SET)
				{
				key_counter ++;
				if(key_counter >= 32)		// 2bo߁H
					{
					key_counter = 0;
//					printf("MOde Set\r\n");
					LCDErase();
					memcpy(LCDText, "ò            Ĺ Ӱ  BAT", 32);
					memcpy(LCDText+10, version, strlen(version));
					disp_flag = 1;
					blink_start = 16;		// Ĺò@_
					blink_number = 3;
					blink_on = 1;
					disp_mode = 1;
					next_state = 2;
					state = 100;
					}
				else
					{
					state = 1;
					}
				}
			else
				{
				state = 0;
				}
			break;

		case 2:							// "Ĺò" _ŏ
			if(key == SET)				// SET L[Ŗ߂
				{
				blink_on = 0;			// _Œ~
				disp_mode = 0;			// Normal \[hݒ
				next_state = 0;
				state = 100;				// L[ItmF@state 0 
				}
			else if(key == SELECT)
				{
				blink_start = 20;		// hӰ޾òh@_ł
				blink_number = 4;
				blink_on = 1;
				next_state = 3;			// 				
				state = 100;
				}
			else if(key == CHANGE)		// "ĹòύXӰ" 
				{
				LCDErase();
				memcpy(LCDText, "   Ĺò", 10);
				youbi_set(1);				// J_\
				disp_flag = 1;
				blink_start = 17;		// hӰ޾òh@_ł
				blink_number = 3;
				blink_on = 1;
				disp_mode = 2;
				next_state = 6;			// L[ItmF
				state = 100;
				}
			else
				{
				state = 2;
				}
			break;		


		case 3:							// Ӱ޾ò_ŏ
			if(key == SET)				// SET L[Ŗ߂
				{
				blink_on = 0;
				disp_mode = 0;
				next_state = 0;
				state = 100;
				}
			else if(key == SELECT)		//@L@_ł
				{
				blink_start = 25;	
				blink_number = 3;
				blink_on = 1;
				next_state = 4;				
				state = 100;
				}
			else if (key == CHANGE)		// Ӱ޾òύXӰނ
				{						// ɕ\e邱ƁI
				read_settei(0,0);		// SUN, TIME1 ̐ݒǂݏo 
				yotei_time = 0;
				LCDErase();
				yotei_1_set();					
				yotei_2_set();
				disp_flag = 1;
				blink_start = 0;	
				blink_number = 3;
				blink_on = 1;
				next_state = 10;
				state = 100;				
				}
			else
				{			
				state = 3;
				}
			break;


		case 4:							// ؾò_ŏ
			if(key == SET)				// SET L[Ŗ߂
				{
				blink_on = 0;
				disp_mode = 0;
				next_state = 0;
				state = 100;
				}
			else if(key == SELECT)		//@@_ł
				{
				blink_start = 29;	
				blink_number = 3;
				blink_on = 1;
				next_state = 5;				
				state = 100;
				}
			else if (key == CHANGE)		// LòύXӰނ
				{						// ɕ\e邱ƁI
				read_dist_settei();		// L1, L2, L3 ̐ݒǂݏo 
				LCDErase();
				dist_1_set();			// l\				
				dist_2_set();			// XVhl\
				disp_flag = 1;
				blink_start = 18;	
				blink_number = 3;
				blink_on = 1;
				disp_mode = 3;
				next_state = 30;		// ؐݒ胂[h
				state = 100;				
				}
			else
				{			
				state = 4;
				}
			break;

		case 5:							// ò_ŏ
			if(key == SET)				// SET L[Ŗ߂
				{
				blink_on = 0;
				disp_mode = 0;
				next_state = 0;
				state = 100;
				}
			else if(key == SELECT)		//@gPC@_ł
				{
				blink_start = 16;	
				blink_number = 3;
				blink_on = 1;
				next_state = 2;				
				state = 100;
				}
			else if (key == CHANGE)		// ݱ\Ӱނ
				{					
				LCDErase();
				next_state =33;			// ݱ\Ӂ[h		
				state = 100;
				}
			else
				{			
				state = 5;
				}
			break;


		case 6:							// Youbi _ŏ
			if(key == SET)				// m[}\֖߂
				{
				blink_on = 0;
				disp_mode = 0;
				next_state = 0;
				state = 100;
				}
			else if (key == SELECT)
				{
				blink_start = 22;		// blink "Hour" 
				blink_number = 2;
				blink_on = 1;
				next_state = 7;
				state = 100;
				}				
			else if(key == CHANGE)
				{
				youbi ++;				// jύXĕ\XV
				if(youbi == 7)
					{
					youbi = 0;
					}
				youbi_set(1);
				disp_flag = 1;
				next_state = 6;
				state = 100;
				}
			else
				{
				state = 6;
				}
			break;

		case 7:							// HOur _ŏ
			if(key == SET)
				{
				blink_on = 0;
				disp_mode = 0;
				next_state = 0;
				state = 100;
				}
			else if (key == SELECT)
				{
				blink_start = 25;		// blink "Min" 
				blink_number = 2;
				blink_on = 1;
				next_state = 8;
				state = 100;
				}				
			else if(key == CHANGE)
				{
				hour ++;				// ύXĕ\XV
				if(hour == 24)
					{
					hour = 0;
					}
				youbi_set(1);
				disp_flag = 1;
				next_state = 7;
				state = 100;			// L[ItmF
				}
			else
				{
				state = 7;
				}
			break;

		case 8:						//  _ŏ
			if(key == SET)
				{
				blink_on = 0;
				disp_mode = 0;
				next_state = 0;
				state = 100;
				}
			else if (key == SELECT)
				{
				blink_start = 28;		// blink "b" 
				blink_number = 3;
				blink_on = 1;
				next_state = 9;
				state = 100;
				}				
			else if(key == CHANGE)
				{
				min ++;				// ύXĕ\XV
				if(min == 60)
					{
					min = 0;
					}
				youbi_set(1);
				disp_flag = 1;
				next_state = 8;
				state = 100;			// L[ItmF
				}
			else
				{
				state = 8;
				}
			break;

		case 9:						// b _ŏ
			if(key == SET)
				{
				blink_on = 0;
				disp_mode = 0;
				next_state = 0;
				state = 100;
				}
			else if (key == SELECT)
				{
				blink_start = 17;		// blink "j" 
				blink_number = 3;
				blink_on = 1;
				next_state = 6;
				state = 100;
				}				
			else if(key == CHANGE)
				{
				sec = 0;				// bZbgĕ\XV
				TMR1H = 0xF8;		//  NbN@16Hz 
				TMR1L = 0x00;		// 
				counter_16Hz = 0;
				int_500ms_flag = 0;
				youbi_set(1);
				disp_flag = 1;
				next_state = 9;
				state = 100;			// L[ItmF
				}
			else
				{
				state = 9;
				}
			break;

		case 10:
			if(key == SET)
				{						// 
				write_settei(yotei_youbi, yotei_time);
				blink_on = 0;
				disp_mode = 0;
				next_state = 0;
				state = 100;				
				}
			else if (key == SELECT)
				{
				blink_start = 4;		// blink "ԃ][" 
				blink_number = 5;
				blink_on = 1;
				next_state = 11;
				state = 100;				
				}
			else if (key == CHANGE)
				{
				write_settei(yotei_youbi, yotei_time);
				yotei_youbi ++;				// \jύXĕ\XV
				if(yotei_youbi == 7)
					{
					yotei_youbi = 0;
					}
				read_settei(yotei_youbi, yotei_time);
				yotei_1_set();
				yotei_2_set();
				disp_flag = 1;
				next_state = 10;
				state = 100;			// L[ItmF
				}
			else
				{
				state = 10;
				}
			break;

		case 11:							// ^C][_
			if(key == SET)
				{
				write_settei(yotei_youbi, yotei_time);
				blink_on = 0;
				disp_mode = 0;
				next_state = 0;
				state = 100;				
				}
			else if (key == SELECT)
				{
				blink_start = 10;		// blink "݃It" 
				blink_number = 2;
				blink_on = 1;
				next_state = 12;
				state = 100;				
				}
			else if (key == CHANGE)
				{
				write_settei(yotei_youbi, yotei_time);
				yotei_time ++;				// \莞ԃXbgύXĕ\XV
				if(yotei_time == 5)
					{
					yotei_time = 0;
					}
				read_settei(yotei_youbi, yotei_time);
				yotei_1_set();
				yotei_2_set();
				disp_flag = 1;
				next_state = 11;
				state = 100;			// L[ItmF
				}
			else
				{
				state = 11;
				}
			break;

		case 12:							// IIt@_
			if(key == SET)
				{
				write_settei(yotei_youbi, yotei_time);
				blink_on = 0;
				disp_mode = 0;
				next_state = 0;
				state = 100;				
				}
			else if (key == SELECT)
				{
				blink_start = 16;		// blink "start time" 
				blink_number = 2;
				blink_on = 1;
				next_state = 13;
				state = 100;				
				}
			else if (key == CHANGE)
				{
				yotei_sw ++;				// \IItXCb`ύXĕ\XV
				if(yotei_sw == 2)
					{
					yotei_sw = 0;
					}
				yotei_1_set();
				yotei_2_set();
				disp_flag = 1;
				next_state = 12;
				state = 100;			// L[ItmF
				}
			else
				{
				state = 12;
				}
			break;

		case 13:
			if(key == SET)
				{
				write_settei(yotei_youbi, yotei_time);
				blink_on = 0;
				disp_mode = 0;
				next_state = 0;
				state = 100;				
				}
			else if (key == SELECT)
				{
				blink_start = 19;		// blink "yotei start min" 
				blink_number = 2;
				blink_on = 1;
				next_state = 14;
				state = 100;				
				}
			else if (key == CHANGE)
				{
				yotei_hour_start ++;				// \jύXĕ\XV
				if(yotei_hour_start == 24)
					{
					yotei_hour_start = 0;
					}
				yotei_1_set();
				yotei_2_set();
				disp_flag = 1;
				next_state = 13;
				state = 100;			// L[ItmF
				}
			else
				{
				state = 13;
				}
			break;

		case 14:
			if(key == SET)
				{
				write_settei(yotei_youbi, yotei_time);
				blink_on = 0;
				disp_mode = 0;
				next_state = 0;
				state = 100;				
				}
			else if (key == SELECT)
				{
				blink_start = 26;		// blink "yotei stop hour" 
				blink_number = 2;
				blink_on = 1;
				next_state = 15;
				state = 100;				
				}
			else if (key == CHANGE)
				{
				yotei_min_start ++;				// \jύXĕ\XV
				if(yotei_min_start == 60)
					{
					yotei_min_start = 0;
					}
				yotei_1_set();
				yotei_2_set();
				disp_flag = 1;
				next_state = 14;
				state = 100;			// L[ItmF
				}
			else
				{
				state = 14;
				}
			break;		

		case 15:
			if(key == SET)
				{
				write_settei(yotei_youbi, yotei_time);
				blink_on = 0;
				disp_mode = 0;
				next_state = 0;
				state = 100;				
				}
			else if (key == SELECT)
				{
				blink_start = 29;		// blink "yotei stop hour" 
				blink_number = 2;
				blink_on = 1;
				next_state = 16;
				state = 100;				
				}
			else if (key == CHANGE)
				{
				yotei_hour_stop ++;				// \jύXĕ\XV
				if(yotei_hour_stop == 24)
					{
					yotei_hour_stop = 0;
					}
				yotei_1_set();
				yotei_2_set();
				disp_flag = 1;
				next_state = 15;
				state = 100;			// L[ItmF
				}
			else
				{
				state = 15;
				}
			break;	
	
		case 16:
			if(key == SET)
				{
				write_settei(yotei_youbi, yotei_time);
				blink_on = 0;
				disp_mode = 0;
				next_state = 0;
				state = 100;				
				}
			else if (key == SELECT)
				{
				LCDErase();
				yotei_1_set();					
				yotei_3_set();
				disp_flag = 1;
				blink_start = 17;		// blink "yotei stop hour" 
				blink_number = 2;
				blink_on = 1;
				next_state = 17;
				state = 100;				
				}
			else if (key == CHANGE)
				{
				yotei_min_stop ++;				// \jύXĕ\XV
				if(yotei_min_stop == 60)
					{
					yotei_min_stop = 0;
					}
				yotei_1_set();
				yotei_2_set();
				disp_flag = 1;
				next_state = 16;
				state = 100;			// L[ItmF
				}
			else
				{
				state = 16;
				}
			break;		
		
		case 17:
			if(key == SET)
				{
				write_settei(yotei_youbi, yotei_time);
				blink_on = 0;
				disp_mode = 0;
				next_state = 0;
				state = 100;				
				}
			else if (key == SELECT)
				{

				blink_start = 20;		// blink "yotei stop hour" 
				blink_number = 2;
				blink_on = 1;
				next_state = 18;
				state = 100;				
				}
			else if (key == CHANGE)
				{
				sensor_sw1 ++;				// XCb`
				if(sensor_sw1 == 2)
					{
					sensor_sw1 = 0;
					}
				yotei_1_set();					
				yotei_3_set();
				disp_flag = 1;
				next_state = 17;
				state = 100;			// L[ItmF
				}
			else
				{
				state = 17;
				}
			break;		

		case 18:
			if(key == SET)
				{
				write_settei(yotei_youbi, yotei_time);
				blink_on = 0;
				disp_mode = 0;
				next_state = 0;
				state = 100;				
				}
			else if (key == SELECT)
				{
				blink_start = 23;		// blink "yotei stop hour" 
				blink_number = 2;
				blink_on = 1;
				next_state = 19;
				state = 100;				
				}
			else if (key == CHANGE)
				{
				sensor_sw2 ++;				// 
				if(sensor_sw2 == 2)
					{
					sensor_sw2 = 0;
					}
				yotei_1_set();					
				yotei_3_set();
				disp_flag = 1;
				next_state = 18;
				state = 100;			// L[ItmF
				}
			else
				{
				state = 18;
				}
			break;		

		case 19:
			if(key == SET)
				{
				write_settei(yotei_youbi, yotei_time);
				blink_on = 0;
				disp_mode = 0;
				next_state = 0;
				state = 100;				
				}
			else if (key == SELECT)
				{

				blink_start = 26;		// blink "yotei stop hour" 
				blink_number = 2;
				blink_on = 1;
				next_state = 20;
				state = 100;				
				}
			else if (key == CHANGE)
				{
				sensor_sw3 ++;				// \jύXĕ\XV
				if(sensor_sw3 == 2)
					{
					sensor_sw3 = 0;
					}
				yotei_1_set();					
				yotei_3_set();
				disp_flag = 1;
				next_state = 19;
				state = 100;			// L[ItmF
				}
			else
				{
				state = 19;
				}
			break;		

		case 20:
			if(key == SET)
				{
				write_settei(yotei_youbi, yotei_time);
				blink_on = 0;
				disp_mode = 0;
				next_state = 0;
				state = 100;				
				}
			else if (key == SELECT)
				{
				blink_start = 29;		// blink "yotei stop hour" 
				blink_number = 2;
				blink_on = 1;
				next_state = 21;
				state = 100;				
				}
			else if (key == CHANGE)
				{
				sensor_sw4 ++;				// \jύXĕ\XV
				if(sensor_sw4 == 2)
					{
					sensor_sw4 = 0;
					}
				yotei_1_set();					
				yotei_3_set();
				disp_flag = 1;
				next_state = 20;
				state = 100;			// L[ItmF
				}
			else
				{
				state = 20;
				}
			break;		

		case 21:
			if(key == SET)
				{
				write_settei(yotei_youbi, yotei_time);
				blink_on = 0;
				disp_mode = 0;
				next_state = 0;
				state = 100;				
				}
			else if (key == SELECT)
				{
				LCDErase();
				yotei_1_set();					
				yotei_2_set();
				disp_flag = 1;
				blink_start = 0;		// blink "yotei stop hour" 
				blink_number = 3;
				blink_on = 1;
				next_state = 10;
				state = 100;				
				}
			else if (key == CHANGE)
				{
				detect_time ++;				// \jύXĕ\XV
				if(detect_time == 11)
					{
					detect_time = 0;
					}
				yotei_1_set();					
				yotei_3_set();
				disp_flag = 1;
				next_state = 21;
				state = 100;			// L[ItmF
				}
			else
				{
				state = 21;
				}
			break;		

		case 30:						// ؾݻ@ݒ
			if(key == SET)
				{
				write_dist_settei();
				blink_on = 0;
				disp_mode = 0;
				next_state = 0;
				state = 100;				
				}
			else if (key == SELECT)
				{
				blink_start = 24;		// blink "yotei stop hour" 
				blink_number = 3;
				blink_on = 1;
				next_state = 31;
				state = 100;				
				}
			else if (key == CHANGE)
				{
				detect1 = detect1 + 10;
				if (detect1 >= 150)
					{
					detect1 = 20;
					}
				LCDErase();
				dist_1_set();			// l\				
				dist_2_set();			// XVhl\
				disp_flag = 1;
				next_state = 30;
				state = 100;			// L[ItmF
				}
			else
				{
				state = 30;
				}
			break;	

		case 31:						// ؾݻ@ݒ
			if(key == SET)
				{
				write_dist_settei();
				blink_on = 0;
				disp_mode = 0;
				next_state = 0;
				state = 100;				
				}
			else if (key == SELECT)
				{
				blink_start = 30;		// blink "yotei stop hour" 
				blink_number = 2;
				blink_on = 1;
				next_state = 32;
				state = 100;				
				}
			else if (key == CHANGE)
				{
				detect2 = detect2 + 10;
				if (detect2 >= 150)
					{
					detect2 = 20;
					}
				LCDErase();
				dist_1_set();			// l\				
				dist_2_set();			// XVhl\
				disp_flag = 1;
				next_state = 31;
				state = 100;			// L[ItmF
				}
			else
				{
				state = 31;
				}
			break;	


		case 32:						// ؾݻ@ݒ
			if(key == SET)
				{
				write_dist_settei();
				blink_on = 0;
				disp_mode = 0;
				next_state = 0;
				state = 100;				
				}
			else if (key == SELECT)
				{
				blink_start = 18;		// blink "yotei stop hour" 
				blink_number = 3;
				blink_on = 1;
				next_state = 30;
				state = 100;				
				}
			else if (key == CHANGE)
				{
				detect3 = detect3 + 10;
				if (detect3 >= 80)
					{
					detect3 = 10;
					}
				LCDErase();
				dist_1_set();			// l\				
				dist_2_set();			// XVhl\
				disp_flag = 1;
				next_state = 32;
				state = 100;			// L[ItmF
				}
			else
				{
				state = 32;
				}
			break;	

		case 33:							// ò_ŏ
			if(key == SET)				// SET L[Ŗ߂
				{
				blink_on = 0;
				disp_mode = 0;
				next_state = 0;
				state = 100;
				}
			else
				{
				itoa(vBatt, 10, tempbuf);			// d\
				memcpy(LCDText+0, "ݱ =", 7);		
				memcpy(LCDText+8, tempbuf, 1);		
				memcpy(LCDText+9, ".", 1);		
				memcpy(LCDText+10, tempbuf+1, 1);			
				memcpy(LCDText+11, "v", 1);		
				state = 33;
				}
			break;


		case 100:
			if(key == OPEN)				// KEY mF
				{
				state = next_state;
				}
			else
				{
				state = 100;
				}
			break;

		default:
			break;
		}
}

void read_dist_settei(void)
{
	int addr;

	addr = 6*30 + 5*6; 
	detect1 = read_eeprom(addr);
	detect2 = read_eeprom(addr+1);
	detect3 = read_eeprom(addr+2);
}



void write_dist_settei(void)
{
	int addr;

	addr = 6*30 + 5*6;
	write_eeprom(addr, detect1);
	write_eeprom(addr+1, detect2);
	write_eeprom(addr+2, detect3);
}

void dist_1_set(void)
{
	LCDText[0] = 'S';
	LCDText[1] = '1';
	LCDText[2] = ':';
	itoa(dist1, 10, tempbuf);
	if(dist1 >= 0 && dist1 <= 9)		// 1̂Ƃ͕\Eɂ炷
		{
		LCDText[3] = ' ';
		LCDText[4] = ' ';		
		memcpy(LCDText+5, tempbuf, 1);	
		}
	else if (dist1 >=10 && dist1 <= 99)
		{
		LCDText[3] = ' ';		
		memcpy(LCDText+4, tempbuf, 2);
		}
	else
		{
		memcpy(LCDText+3, tempbuf, 3);
		}
	LCDText[9] = 'S';
	LCDText[10] = '2';
	LCDText[11] = ':';
	itoa(dist2, 10, tempbuf);
	if(dist2 >= 0 && dist2 <= 9)		// 1̂Ƃ͕\Eɂ炷
		{
		LCDText[12] = ' ';
		LCDText[13] = ' ';		
		memcpy(LCDText+14, tempbuf, 1);	
		}
	else if (dist2 >=10 && dist2 <= 99)
		{
		LCDText[12] = ' ';		
		memcpy(LCDText+13, tempbuf, 2);
		}
	else
		{
		memcpy(LCDText+12, tempbuf, 3);
		}
}

void dist_2_set(void)
{
	LCDText[16] = 'A';
	LCDText[17] = ':';
	itoa(detect1, 10, tempbuf);
	if(detect1 >= 0 && detect1 <= 9)		// 1̂Ƃ͕\Eɂ炷
		{
		LCDText[18] = ' ';
		LCDText[19] = ' ';		
		memcpy(LCDText+20, tempbuf, 1);	
		}
	else if (detect1 >=10 && detect1 <= 99)
		{
		LCDText[18] = ' ';		
		memcpy(LCDText+19, tempbuf, 2);
		}
	else
		{
		memcpy(LCDText+18, tempbuf, 3);
		}

	LCDText[22] = 'B';
	LCDText[23] = ':';
	itoa(detect2, 10, tempbuf);
	if(detect2 >= 0 && detect2 <= 9)		// 1̂Ƃ͕\Eɂ炷
		{
		LCDText[24] = ' ';
		LCDText[25] = ' ';		
		memcpy(LCDText+26, tempbuf, 1);	
		}
	else if (detect2 >=10 && detect2 <= 99)
		{
		LCDText[24] = ' ';		
		memcpy(LCDText+25, tempbuf, 2);
		}
	else
		{
		memcpy(LCDText+24, tempbuf, 3);
		}

	LCDText[28] = 'C';
	LCDText[29] = ':';
	itoa(detect3, 10, tempbuf);
	if(detect3 >= 0 && detect3 <= 9)		// 1̂Ƃ͕\Eɂ炷
		{
		LCDText[30] = ' ';
		memcpy(LCDText+32, tempbuf, 1);	
		}
	else
		{
		memcpy(LCDText+30, tempbuf, 2);
		}
}



void read_settei(int youbi, int time)
{
	int addr, temp1;

	addr = (youbi * 30)+ (time * 6);
	yotei_hour_start = read_eeprom(addr);
	yotei_min_start	 = read_eeprom(addr+1);		
	yotei_hour_stop	 = read_eeprom(addr+2);		
	yotei_min_stop	 = read_eeprom(addr+3);
	temp1			 = read_eeprom(addr+4);
	detect_time		 = read_eeprom(addr+5);

	if((temp1 & 0x80) == 0x00)
		yotei_sw = 0;
	else
		yotei_sw = 1;

	if((temp1 & 0x01) == 0x00)
		sensor_sw1 = 0;
	else
		sensor_sw1 = 1;

	if((temp1 & 0x02) == 0x00)
		sensor_sw2 = 0;
	else
		sensor_sw2 = 1;		

	if((temp1 & 0x04) == 0x00)
		sensor_sw3 = 0;
	else
		sensor_sw3 = 1;

	if((temp1 & 0x08) == 0x00)
		sensor_sw4 = 0;
	else
		sensor_sw4 = 1;

}

void write_settei(int youbi, int time)
{
	int addr, temp1;

	temp1 = 0;
	if(yotei_sw == 1)
		temp1 = temp1 | 0x80;
	if(sensor_sw1 == 1)
		temp1 = temp1 | 0x01;
	if(sensor_sw2 == 1)
		temp1 = temp1 | 0x02;
	if(sensor_sw3 == 1)
		temp1 = temp1 | 0x04;
	if(sensor_sw4 == 1)
		temp1 = temp1 | 0x08;

	addr = (youbi * 30)+ (time * 6);
	write_eeprom(addr, yotei_hour_start);	
	write_eeprom(addr+1, yotei_min_start);
	write_eeprom(addr+2, yotei_hour_stop);
	write_eeprom(addr+3, yotei_min_stop);
	write_eeprom(addr+4, temp1);
	write_eeprom(addr+5, detect_time);
}



void yotei_1_set(void)
{
	switch(yotei_youbi)
		{
		case 0: 	// SUN
			memcpy(LCDText, "SUN", 3);
			break;
		case 1:		// MON
			memcpy(LCDText, "MON", 3);
			break;
		case 2:
			memcpy(LCDText, "TUE", 3);
			break;
		case 3:
			memcpy(LCDText, "WED", 3);
			break;
		case 4:
			memcpy(LCDText, "THU", 3);
			break;
		case 5:
			memcpy(LCDText, "FRI", 3);
			break;
		case 6:
			memcpy(LCDText, "SAT", 3);
			break;
		default:
			break;
		}
	switch(yotei_time)
		{
		case 0:		// Time 1
			memcpy(LCDText+4, "TIME1", 5);
			break;
		case 1:		// Time 2
			memcpy(LCDText+4, "TIME2", 5);
			break;
		case 2:		// Time 3
			memcpy(LCDText+4, "TIME3", 5);
			break;
		case 3:		// Time 4
			memcpy(LCDText+4, "TIME4", 5);
			break;
		case 4:		// Time 5
			memcpy(LCDText+4, "TIME5", 5);
			break;
		default:
			break;
		}
	switch(yotei_sw)
		{
		case 0:
			memcpy(LCDText+10, "", 2);
			break;
		case 1:
			memcpy(LCDText+10, "", 2);
			break;
		default:
			break;
		}

}	

void yotei_2_set(void)
{
	itoa(yotei_hour_start, 10, tempbuf);
	if(yotei_hour_start >= 0 && yotei_hour_start <= 9)		// 1̂Ƃ͕\Eɂ炷
		{
		LCDText[16] = '0';
		memcpy(LCDText+17, tempbuf, 1);	
		}
	else
		{
		memcpy(LCDText+16, tempbuf, 2);
		}
	LCDText[18] = ':';

	itoa(yotei_min_start, 10, tempbuf);
	if(yotei_min_start >= 0 && yotei_min_start <= 9)		// 1̂Ƃ͕\Eɂ炷
		{
		LCDText[19] = '0';
		memcpy(LCDText+20, tempbuf, 1);	
		}
	else
		{
		memcpy(LCDText+19, tempbuf, 2);
		}
	LCDText[22] = '>';
	LCDText[24] = '>';

	itoa(yotei_hour_stop, 10, tempbuf);
	if(yotei_hour_stop >= 0 && yotei_hour_stop <= 9)		// 1̂Ƃ͕\Eɂ炷
		{
		LCDText[26] = '0';
		memcpy(LCDText+27, tempbuf, 1);	
		}
	else
		{
		memcpy(LCDText+26, tempbuf, 2);
		}
	LCDText[28] = ':';

	itoa(yotei_min_stop, 10, tempbuf);
	if(yotei_min_stop >= 0 && yotei_min_stop <= 9)		// 1̂Ƃ͕\Eɂ炷
		{
		LCDText[29] = '0';
		memcpy(LCDText+30, tempbuf, 1);	
		}
	else
		{
		memcpy(LCDText+29, tempbuf, 2);
		}
}


void yotei_3_set(void)
{
	switch(sensor_sw1)
		{
		case 0:
			memcpy(LCDText+17, "", 2);
			break;
		case 1:
			memcpy(LCDText+17, "", 2);
			break;
		default:
			break;
		}

	switch(sensor_sw2)
		{
		case 0:
			memcpy(LCDText+20, "", 2);
			break;
		case 1:
			memcpy(LCDText+20, "", 2);
			break;
		default:
			break;
		}
			
	switch(sensor_sw3)
		{
		case 0:
			memcpy(LCDText+23, "", 2);
			break;
		case 1:
			memcpy(LCDText+23, "", 2);
			break;
		default:
			break;
		}

	switch(sensor_sw4)
		{
		case 0:
			memcpy(LCDText+26, "", 2);
			break;
		case 1:
			memcpy(LCDText+26, "", 2);
			break;
		default:
			break;
		}

	itoa(detect_time, 10, tempbuf);					// m
	if(detect_time >= 0 && detect_time <= 9)		// 1̂Ƃ͕\Eɂ炷
		{
		LCDText[29] = '0';
		memcpy(LCDText+30, tempbuf, 1);	
		}
	else
		{
		memcpy(LCDText+29, tempbuf, 2);
		}	
}
	


void youbi_set(int line)
{
	int offset;
	
	offset = line*16;

		switch(youbi)
			{
			case 0: 	// SUN
				memcpy(LCDText+offset+1, "SUN:", 4);
				break;
			case 1:		// MON
				memcpy(LCDText+offset+1, "MON:", 4);
				break;
			case 2:
				memcpy(LCDText+offset+1, "TUE:", 4);
				break;
			case 3:
				memcpy(LCDText+offset+1, "WED:", 4);
				break;
			case 4:
				memcpy(LCDText+offset+1, "THU:", 4);
				break;
			case 5:
				memcpy(LCDText+offset+1, "FRI:", 4);
				break;
			case 6:
				memcpy(LCDText+offset+1, "SAT:", 4);
				break;
			default:
				break;
			}
		itoa(hour, 10, tempbuf);
		if(hour >= 0 && hour <= 9)		// 1̂Ƃ͕\Eɂ炷
			{
			LCDText[offset+6] = '0';
			memcpy(LCDText+offset+7, tempbuf, 1);
			}
		else
			{
			memcpy(LCDText+offset+6, tempbuf, 2);
			}
		LCDText[offset+8] = ':';
		itoa(min, 10, tempbuf);
		if(min >= 0 && min <= 9)		// 1̂Ƃ͕\Eɂ炷
			{
			LCDText[offset+9] = '0';
			memcpy(LCDText+offset+10, tempbuf, 1);
			}
		else
			{
			memcpy(LCDText+offset+9, tempbuf, 2);
			}

		LCDText[offset+11] = ':';
		itoa(sec, 10, tempbuf);
		if(sec >= 0 && sec <= 9)		// 1̂Ƃ͕\Eɂ炷
			{
			LCDText[offset+12] = '0';
			memcpy(LCDText+offset+13, tempbuf, 1);
			}
		else
			{
			memcpy(LCDText+offset+12, tempbuf, 2);
			}

		if((Lo_Batt_flag == 1) && (line == 0))		// Lo battery ɂ"*" }[N_
			{
			memcpy(LCDText+offset+15, "*", 1);
			}
		else
			{
			memcpy(LCDText+offset+15, " ", 1);
			}			
}




void NormalDisp(void)
{
	if(disp_mode == 0)		// Normal Display
		{
		LCDErase();
		youbi_set(0);

		memcpy (LCDText+16, "ޮ", 6);
		switch(joutai)
			{
			case 0:
				memcpy (LCDText+23, "̻޲", 4);
				break;
			case 1:
				memcpy (LCDText+23, "", 5);
				break;
			case 2:
				memcpy (LCDText+23, "޶", 8);
				break;
			case 3:
				memcpy (LCDText+23, "", 3);
				break;
			case 4:
				memcpy (LCDText+23, "ĳ", 4);
				break;
			case 5:
				memcpy (LCDText+23, "", 7);
				break;

			default:
				break;
			}	
		}
	else if (disp_mode == 2)			// vݒ胂[h\ŎvCNg
		{
		LCDErase();
		memcpy(LCDText, "   Ĺò",10 );
		youbi_set(1);				// J_\
		}		
	else if (disp_mode == 3)			// ݒ胂[h\ŋ\
		{
		LCDErase();
		dist_1_set();
		dist_2_set();
		}		
}



void blink(void)				// LCD _ŏ
{
	int i;

	if(int_500ms_flag ==1)
		{
		int_500ms_flag = 0;
		if (blink_on ==1)
			{
			if(blink_flag == 1)			// Blink Flag = 1 炻̂܂ܕ\
				{
				blink_flag = 0;
				}
			else						// Blink Flag = 0 numberuNŖ߂
				{
				blink_flag = 1;
				memcpy(blink_buf, LCDText + blink_start, blink_number);			// blink_buf ɓ_ŕnumberޔ
				for (i=0; i< blink_number; i++)
					{
					LCDText[blink_start+i] = ' ';				// Xy[XŖ߂
					}
				blink_buf_flag = 1;		// ޔtOZbg
				}	
			}
			disp_flag = 1;
		}
}


void LCDDisp(void)				// LCD \
{
	int i;

	if (disp_flag ==1)
		{

		disp_flag = 0;
		LCDUpdate();					// LCD \
		if(blink_buf_flag == 1)			// ޔtOĂ_ŗpɃobt@
			{
			blink_buf_flag = 0;
			for (i=0; i< blink_number; i++)
				{
				LCDText[blink_start+i] = blink_buf[i];
				}
			}
		}
	if (DCIN == 0)						// LCD Back Light Control
		{
		LCD_BKLGT = 1;
//		printf("DCIN = 0\r\n");
		}
	else
		{
		LCD_BKLGT = 0;
//		printf("DCIN = 1\r\n");
		}		
}
 	



void adc_init(void)			// ADC 
{
	KEY_VREF = 1;		// ANS2
	KEY_IN = 1;		// ANS9
	SENSOR2 = 1;		// ANS11 
	SENSOR1 = 1;		// ANS13

	ADON = 1;				// ADC ON
	ADFM = 1;				// Right Justified
	VCFG0 = 0;				// VDD
	VCFG1 = 0;				// VSS
	FVREN = 1;				// Fixed 0.6V Vref ON
	ADCS1 = 0;				// AD clock = FOSC/8
	ADCS0 = 0;
}


int read_key(void)			// L[͔菈
{
	ADON = 1;
	CHS3 = 1;		// ANS9
	CHS2 = 0;
	CHS1 = 0;
	CHS0 = 1;
	delay_ms(1);
	GODONE = 1;		// ADC Start
	while(GODONE == 1);
	vKey = ADRESH;
	vKey = (vKey << 8) | ADRESL;		// Sensor Voltage value
	if(vKey >=0 && vKey < 400)
		{
//		printf("key1\r\n");
		return 1;
		}
	else if(vKey >= 400 && vKey < 600)
		{
//		printf("key2\r\n");
		return 2;
		}
	else if (vKey >= 600 && vKey < 800)
		{
//		printf("key3\r\n");
		return 3;
		}
	else
		{
		return 0;
		}
}




void calendar(void)
{
	int_1s_flag = 0;
//	sec++;
//	printf("sec = %d\r\n", sec);
	if(sec==60)
		{
		sec=0;
		min++;
		if(min==60)
			{
			min=0;
			hour++;
			if(hour == 24)
				{
				hour = 0;
				youbi++;
				if(youbi==7)
					{
					youbi = 0;
					}
				}
			}
		}
}
	

//  2.5 <= Vo            Error
//  1.5 <= Vo < 2.5      D = -20Vo + 70   
//  1.0 <= Vo < 1.5      D = -40Vo + 80
//  0.75 <= Vo < 1.0     D = -120Vo + 180
//  0.5 <= Vo < 0.75     D = -160Vo + 210
//  Vo < 0.5             Error


///////// LCD Library@@/////////

void LCDInit(void)
{
	char i;

	memset(LCDText, ' ', sizeof(LCDText)-1);
	LCDText[sizeof(LCDText)-1] = 0;
	// Wait the required time for the LCD to reset
	delay_ms(40);
	RS = 0;
	PORTA = (PORTA & 0x0f) | 0x30;		// 8rbg[h̏o
	delay_us(1);					// Wait Data setup time (min 40ns)
	for(i = 0; i < 3; i++)		// 3o
	{
		ENBL = 1;
		delay_us(1);			// Wait E Pulse width time (min 230ns)
		ENBL = 0;
		delay_ms(5);			// ԑ҂
	}
	PORTA = (PORTA & 0x0f) | 0x20;		// 4rbg[ho
	delay_us(10);					// Wait Data setup time (min 40ns)
	ENBL = 1;
	delay_us(1);				// Wait E Pulse width time (min 230ns)
	ENBL = 0;
	delay_us(100);				// ԑ҂
	// Set the entry mode
	LCDWrite(0, 0x06);		// Increment after each write, do not shift
	delay_us(100);
	// Set the display control
	LCDWrite(0, 0x0C);		// Turn display on, no cusor, no cursor blink

	delay_us(100);
	// Clear the display
	LCDWrite(0, 0x01);	
	delay_ms(2);
}


void LCDUpdate(void)
{
	char i;

	// Go home
	LCDWrite(0, 0x02);			// z[R}ho
	delay_ms(2);					// ԑ҂
	// Output first line
	for(i = 0; i < 16; i++)
	{
		LCDWrite(1, LCDText[i]);
		delay_us(10);
	}
	
	// Set the address to the second line

	LCDWrite(0, 0xC0);
	delay_us(50);
	// Output second line
	for(i = 16; i < 32; i++)
	{
		LCDWrite(1, LCDText[i]);
		delay_us(10);
	}

}


void LCDErase(void)
{
	memset(LCDText, ' ', 32);
}


void LCDWrite(char cd, char Data)
{
	PORTA = (PORTA & 0x0f) | (Data & 0xf0);		// 4rbgo
	RS = cd;
	delay_us(10);		
	ENBL = 1;
	delay_us(10);
	ENBL = 0;	
	delay_us(10);
	/// Lower
	PORTA = (PORTA & 0x0f) | ((Data << 4)& 0xf0) ;	// 4rbgo
	RS = cd;
	delay_us(10);
	ENBL = 1;
	delay_us(10);
	ENBL = 0;	
	delay_us(10);
	RS = 0;
}

